#!/bin/bash

rc=1
# turn off posix which happens if invoked as "sh"
set +o posix
#
# check the command line args. By default we create the archive name
# but the caller can pass it in on the command line
#

source /etc/profile
set -o nounset -o allexport
shopt -s extglob

SCRIPT_TYPE=BACKUP
SCRIPTS_DIR=$VSP_HOME/backup/scripts
cmdargs="$@"
source $SCRIPTS_DIR/common.sh
ARCHIVE_FILE=backup$ARCHIVE_NAME
VMSETS=""
unset UPGRADE

function usage {
echo "Usage: backup.sh [-f archivename] [-k number_backups_to_keep]"
}

NUM_KEEP=0

# process command line arguments
while getopts "f:k:u?v:" flag
do
    case $flag in
        f) ARCHIVE_FILE=$OPTARG;;
        k) if [[ "$OPTARG" != [1-9]*([0-9]) && "$OPTARG" != 0 ]] ; then
               echo "$OPTARG is invalid for number of backups"
               usage
               exit 1
           fi
           NUM_KEEP=$OPTARG
           ;;
        u) UPGRADE=1;;
        v) VMSETS=$OPTARG;;
        *) usage; exit 2;;
    esac
done

# make sure the archive dir exists.
if [ ! -w $VSP_ARCHIVE_DIR ]
then 
    echo "archive directory ($VSP_ARCHIVE_DIR) does not exist or not writeable"
    exit 5
fi


# check if an archive with the specified name already exists
if [ -e $VSP_ARCHIVE_DIR/$ARCHIVE_FILE ]
then
   echo "Archive file name $ARCHIVE_FILE already exists in $VSP_ARCHIVE_DIR"
   exit 3
fi

exec 3> >(exec $VSP_HOME/util/logger.py -f $SCRIPTS_DIR/log.conf)
LOGGER_PID=$!

#
# Ok, we got this far so let's start the backup process.

log "Starting backup --------------------------------------------------------------"
log "Arguments: $cmdargs"
log "Archive set to $ARCHIVE_FILE"
logger -t VSP "$SCRIPT_TYPE started with archive $ARCHIVE_FILE"

BACKUP_ROOT=$TMP_BACKUP_DIR/${ARCHIVE_FILE%.*}
#set trap to remove backup directory when we're done
trap '{ killed ; terminate ; }' INT TERM
trap '{ terminate ; }' EXIT
mkdir -p $BACKUP_ROOT
chmod 777 $BACKUP_ROOT

cd $SCRIPTS_DIR

check_scripts backup.sh

# capture VSP version info
cp -f $VSP_HOME/version/vsp.version $BACKUP_ROOT
$VSP_HOME/bin/swversion > $BACKUP_ROOT/swversion

for SETNAME in $BACKUP_SETS ; do
   EXPAND=$SETNAME/expand.sh
   if [ -f $EXPAND ] ; then
     BACKUP_INSTANCES=$($EXPAND $VMSETS)
   else
     BACKUP_INSTANCES=$SETNAME
   fi
   FILENAME=$SETNAME/backup.sh
   for BACKUP_INSTANCE in $BACKUP_INSTANCES ; do
     BACKUP_SET=$BACKUP_ROOT/$SETNAME
     mkdir -p $BACKUP_SET
     chmod 777 $BACKUP_SET
     if [ -f $EXPAND ] ; then
       BACKUP_SET=$BACKUP_ROOT/$SETNAME/$BACKUP_INSTANCE
       mkdir -p $BACKUP_SET
       chmod 777 $BACKUP_SET
     fi
     VERSION_FILE=$BACKUP_SET/.version
     echo $SETNAME > $BACKUP_SET/.setname
     #run scripts with errexit and nounset options
     log "Executing $FILENAME for $BACKUP_INSTANCE"
     rc=1
     cd $SETNAME

     run_subscript ./backup.sh

     cd - >/dev/null
     if [ $rc -eq 0 ] ; then
        log "$FILENAME - Successful"
     else
        log "$FILENAME - Failed with exit code $rc"
        exit $rc
     fi
   done
done

cd $TMP_BACKUP_DIR
tar zcf $VSP_ARCHIVE_DIR/$ARCHIVE_FILE * >&3 2>&3
rc=$?
if [ $rc -ne 0 ] ; then
   rm -f $VSP_ARCHIVE_DIR/$ARCHIVE_FILE
   log "backup failed at tar with exit code $rc"
   exit $rc
fi

log "$ARCHIVE_FILE saved in $VSP_ARCHIVE_DIR"

#Remove excess backups if NUM_KEEP is set
if [[ $NUM_KEEP > 0 ]] ; then
    cd $VSP_ARCHIVE_DIR
    numremove=$(( $(ls|wc -l) - $NUM_KEEP ))
    if [[ $numremove > 0 ]] ; then
        ls -t | tail -$numremove | xargs rm -f
    fi
fi
