#!/bin/bash

source $SCRIPTS_DIR/config
[[ -f $VSP_BACKUP_DIR/config ]] && source $VSP_BACKUP_DIR/config

# Don't use the default SP PS4 which adds timestamps, since log file already has them.
PS4="+ "
BACKUP_SETS="cdom-keys cdom-host-key cdom-net dom0-keys dom0-host-key dom0-net ldap license ha perf smgr-spirit spirit system template vm-config vm-data webconsole cdom-security dom0-security pmt"
DEFAULT_RESTORE_SETS="ldap license smgr-spirit spirit system vm-data webconsole"
VSP_BACKUP_LOG=$VSP_BACKUP_DIR/backup.log
TMP_BACKUP_DIR=$(mktemp -d)
chmod 755 $TMP_BACKUP_DIR

DATE=$(date +%Y_%m_%d_%H_%M_%S)
ARCHIVE_NAME=_$(hostname -s)_$DATE.tgz

#licensing
WEBLM_FROM=$CATALINA_HOME/webapps/WebLM

#logs
LOGBASE=${LOGBASE:-/var/log}

# Set timeout value
if [ $SCRIPT_TYPE = BACKUP ] ; then
  SCRIPT_TIMEOUT=$BACKUP_TIMEOUT
else
  SCRIPT_TIMEOUT=$RESTORE_TIMEOUT
fi
SCRIPT_TIMEOUT_PID=""

# ensure archive directory exists
if [[ ! -d $VSP_ARCHIVE_DIR ]] ; then
  mkdir -p $VSP_ARCHIVE_DIR
  chmod 755 $VSP_ARCHIVE_DIR
fi

#miscellaneous functions
# wait for a command to succeed
# param 1 is the command to run
# param 2 is how long to wait (default 60 seconds)
# return TRUE if command succeeded
#        FALSE if time exceeded
function waitcommand {
  ((x=0))
  while ! eval $1 && (((x+=1)<${2:-60})); do
    sleep 1
  done
  sleep 2
  eval $1
  return $?
}

# wait for a process to appear in the process table
# param 1 is the process name
# param 2 is how long to wait (defaulted to waitcommand default)
# return TRUE if process appears
#        FALSE if timc exceeded
function waitproc {
  waitcommand "pgrep $1 &>/dev/null" ${2:-60}
  return $?
}

# wait for a process to disappear from the process table
function waitnoproc {
  waitcommand "! pgrep $1 &>/dev/null" ${2:-60}
  return $?
}

# check scripts all exist
# parm 1 is the script name
function check_scripts
{
  for SETNAME in $BACKUP_SETS
  do
    SCRIPT=$SETNAME/$1
    if [ ! -r $SCRIPT ] ; then
      echo "Error: could not find $SCRIPT script"
      rc=1
      exit $rc
    fi
  done
}

# run a subscript with a time limit
# param 1: name of script

function run_subscript {
   # start timer (close file descriptors first)
   setsid bash -c "sleep $SCRIPT_TIMEOUT ; kill -SIGUSR1 $$ " >&- 2>&- 3>&- &
   SCRIPT_TIMEOUT_PID=$!
   echo sleep pid is $SCRIPT_TIMEOUT_PID >&3

   setsid bash -eux -o pipefail $1 >&3 2>&3 &
   SCRIPT_PID=$!
   echo $1 pid is $SCRIPT_PID >&3
   wait $SCRIPT_PID
   rc=$?
   echo $SCRIPT_PID finished >&3
   SCRIPT_PID=""

   # wait to make sure timeout script has started before trying to kill
   sleep .1
   # cancel timer
   echo kill -- -$SCRIPT_TIMEOUT_PID >&3
   kill -- -$SCRIPT_TIMEOUT_PID 2>/dev/null
   SCRIPT_TIMEOUT_PID=""
}

#send messages to logger.py and stdout
function log
{
  echo $SCRIPT_TYPE "$*" >&3
  echo $SCRIPT_TYPE "$*"
}

# write status file of last 10 results for the current
# script type (backup or restore)
# params:
#  $1: SUCCESS or FAILURE

function writeStatus
{
  local stat_file=$VSP_BACKUP_DIR/${SCRIPT_TYPE}_STATUS
  { tail -99 $stat_file 2>/dev/null
    echo $1 $(date) $ARCHIVE_FILE
  } > $stat_file.1
  mv $stat_file.1 $stat_file
  chmod 644 $stat_file
}

# trap called when we are interrupted or killed
function killed
{
   # in this case, set rc to 1 -- can never be a success
   log "Killed"
   rc=1
}

# termination script to run at end of backup or restore
function terminate
{
  cd
  [[ -n "$TMP_BACKUP_DIR" ]] && rm -fr $TMP_BACKUP_DIR
  # cancel script if still running
  if [[ -n "$SCRIPT_PID" ]] ; then
    echo "kill -9 -- -$SCRIPT_PID" >&3
    kill -- -$SCRIPT_PID 2>/dev/null
    sleep .5
    kill -9 -- -$SCRIPT_PID 2>/dev/null
  fi
  # cancel any remaining timer
  if [[ -n "$SCRIPT_TIMEOUT_PID" ]] ; then
    echo kill -- -$SCRIPT_TIMEOUT_PID >&3
    kill -- -$SCRIPT_TIMEOUT_PID 2>/dev/null
  fi
  if [ $rc -eq 0 ] ; then
    rm -f $VSP_BACKUP_DIR/failure
    logger -t VSP "$SCRIPT_TYPE succeeded with archive $ARCHIVE_FILE"
    log "****** Succeeded ******"
    writeStatus SUCCESS
  else
    [[ BACKUP = "$SCRIPT_TYPE" ]] && rm -f $VSP_ARCHIVE_DIR/$ARCHIVE_FILE
    echo $SCRIPT_TYPE > $VSP_BACKUP_DIR/failure
    chmod 644 $VSP_BACKUP_DIR/failure
    logger -t VSP -p syslog.err "A scheduled or on-demand $SCRIPT_TYPE has FAILED - check /vspdata/backup/backup.log in CDom for details"
    log "****** FAILED ******"
    writeStatus FAILURE
  fi
  # close output to logger
  exec 3>&-
  # wait for logger to process remaining output
  sleep 1
  # now ensure that the logger process goes away
  kill $LOGGER_PID 2>/dev/null
  trap - INT TERM EXIT
  exit $rc
}

function timeout {
  rc=10
  log "Timer of value $SCRIPT_TIMEOUT expired."

  # get rid of message about killed process
  exec 2>/dev/null
  kill -9 -- -$SCRIPT_PID 2>/dev/null 
  exit $rc
}
trap timeout SIGUSR1

# Recursively copy a list of files into the backup set
# Invokes the files.sh script to retrieve the list of files
function copy_files
{
  mkdir -p $BACKUP_SET/copy_files
  files=$BACKUP_SET/.filelist
  optfiles=$BACKUP_SET/.optfilelist
  [ -x files.sh ] && ./files.sh > $files
  if [ -x optfiles.sh ] ; then
    ./optfiles.sh > $optfiles
    if [ -s $optfiles ] ; then
      ls -d $(<$optfiles) >>$files 2>/dev/null || :
    fi
  fi
  if [[ -s $files ]] ; then
    file_list=$(mktemp)
    # do not directly pipe find to cpio.  We want any missing files
    # or directories to constitute a failure
    find $(<$files) -depth -print > $file_list
    cpio -pamd $BACKUP_SET/copy_files < $file_list
    rm -f $file_list
  fi
}

# Recursively copy a list of files from dom0 into the backup set
# Invokes the files.sh script to retrieve the list of files
function copy_files_dom0
{

  # turn pathname expansion off, so it happens on dom0
  set -f
  mkdir -p $BACKUP_SET/copy_files_dom0
  files=$BACKUP_SET/.filelist_dom0
  optfiles=$BACKUP_SET/.optfilelist_dom0
  [ -x files_dom0.sh ] && ./files_dom0.sh > $files
  if [ -x optfiles_dom0.sh ] ; then
    ./optfiles_dom0.sh > $optfiles
    if [ -s $optfiles ] ; then
      ssh dom0.vsp ls -d $(<$optfiles) >>$files 2>/dev/null || :
    fi
  fi
  if [[ -s $files ]] ; then
    file_list=$(mktemp)
    # do not directly pipe find to cpio.  We want any missing files
    # or directories to constitute a failure
    ssh dom0.vsp find $(<$files) -depth -print > $file_list
    cd $BACKUP_SET/copy_files_dom0
    ssh dom0.vsp cpio -o < $file_list | cpio -imd 2>/dev/null
    rm -f $file_list
  fi
  # turn pathname expansion back on
  set +f
}

# Restore files from a previous copy_files
function restore_files
{
  files=$BACKUP_SET/.filelist
  optfiles=$BACKUP_SET/.optfilelist
  [[ ! -d $BACKUP_SET/copy_files ]] && return
  cd $BACKUP_SET/copy_files
  [ -s $optfiles ] && rm -fr $(<$optfiles)
  if [ -s $files ] ; then
    rsync -a --delete --relative --no-implied-dirs $(cut -c2- $files) /
  fi
}

# Restore files from a previous copy_files
function restore_files_dom0
{
  files=$BACKUP_SET/.filelist_dom0
  optfiles=$BACKUP_SET/.optfilelist_dom0
  [[ ! -d $BACKUP_SET/copy_files_dom0 ]] && return
  cd $BACKUP_SET/copy_files_dom0
  # turn pathname expansion off, so it happens on dom0
  set -f
  [ -s $optfiles ] && ssh dom0.vsp rm -fr $(<$optfiles)
  # turn pathname expansion back on
  set +f
  if [ -s $files ] ; then
    rsync -a --delete --relative --no-implied-dirs $(cut -c2- $files) dom0.vsp:/
  fi
}
